package com.gzlh.bus;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.gzlh.config.dto.CaputreSetting;
import com.gzlh.config.dto.SerialSetting;
import com.gzlh.device.capture.factory.CaptureFactory;
import com.gzlh.config.ModuleEnum;
import com.gzlh.config.SystemObject;
import com.gzlh.device.electron.factory.ElectronFactory;

import com.gzlh.device.led.factory.LedFactory;
import com.gzlh.device.led.utils.LedOptions;
import com.gzlh.device.plc.event.PLCEvent;
import com.gzlh.device.plc.handler.PLCHadnler;
import com.gzlh.device.weighbridge.action.WeighbridgeCommonAction;
import com.gzlh.device.weighbridge.factory.WeighbridgeFactory;
import com.gzlh.device.weighbridge.handler.impl.CommonWeighbridgeHandler;
import com.gzlh.entity.CommandBO;
import com.gzlh.entity.ReqBO;
import com.gzlh.utils.DeviceCache;
import com.gzlh.utils.WordHandlerUtils;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
public class EventThread implements Runnable{
    private String eventName;

    public EventThread(String eventName) {
        this.eventName=eventName;
    }

    /**
     * 处理事件
     *
     * @param eventName 服务.事件名 比如led.ready
     */
    public void handlerEvent(String eventName) {
        SerialSetting serialSetting= SysConfig.serialSetting;
        LedFactory ledFactory= SystemObject.ledFactory;
        WeighbridgeFactory weighbridgeFactory=SystemObject.weighbridgeFactory;
        ElectronFactory electronFactory= SystemObject.electronFactory;
        PLCHadnler plcHadnler= SpringUtil.getBean(PLCHadnler.class);
        CaptureFactory captureFactory=SystemObject.captureFactory;
        CaputreSetting caputreSetting=SysConfig.caputreSetting;
      //  log.info("event:{}", eventName);
        SysConfig.eventDTOList.stream().filter(eventDTO -> StrUtil.equals(eventName, eventDTO.getName()))
                .findFirst().ifPresent(eventDTO -> {
            List<String> actionList = eventDTO.getActionList().getAction();
            //依次执行动作
            for (String action:actionList) {
                String module = StrUtil.subBefore(action, ".", true);
                String command = StrUtil.subAfter(action, ".", true);
//                中断事件动作
                if (DeviceCache.isInterrupt()){
                    //停止读数
                    CommonWeighbridgeHandler.isStart=false;
                    DeviceCache.setInterrupt(false);
                    break;
                }
                if (StrUtil.equals(module, ModuleEnum.INFRARED_MODULE.getModuleEn())) {
                    //动作属于红外
                } else if (StrUtil.equals(module, ModuleEnum.LED_MODULE.getModuleEn())) {
                    //动作属于led
                    ledFactory.handler(serialSetting.getLed().getBrand()).handlerAction(command);
                } else if (StrUtil.equals(module, ModuleEnum.WEIGHBRIDGE_MODULE.getModuleEn())) {
                    //动作属于地磅
                    weighbridgeFactory.handler(serialSetting.getWeighbridge().getBrand()).handlerAction(command);
                }else if (StrUtil.equals(module, ModuleEnum.CAPTURE_MODULE.getModuleEn())) {
                    //动作属于抓拍单元--车牌识别
                    captureFactory.handler(caputreSetting.getBrand()).handlerAction(command);
                }else if (StrUtil.equals(module, ModuleEnum.PLC_MODULE.getModuleEn())) {
                    //动作属于plc
                    plcHadnler.handlerAction(command);
                }else if (StrUtil.equals(module, ModuleEnum.ELECTRON_MODULE.getModuleEn())) {
                    //动作属于电子车牌
                    electronFactory.handler(serialSetting.getElectron().getBrand()).handlerAction(command);
                }   else if (StrUtil.equals(module, ModuleEnum.CENTER_MODULE.getModuleEn())) {
                    //动作属于中心总线
                    handlerAction(command);
                }
                ThreadUtil.sleep(250);
            }
        });

    }

    /**
     * 调度中心命令处理
     *
     * @param action
     */
    private void handlerAction(String action) {
        if (StrUtil.contains(action, CenterActionCommand.SLEEP_COMMAND)) {
            //休眠
            handlerSleep(action);
        }else if (StrUtil.contains(action, CenterActionCommand.SUBMIT)){
            handlerSubmit();
        }
    }

    /**
     * 调度中心休眠
     *
     * @param action
     */
    private void handlerSleep(String action) {
        String time = StrUtil.subBefore(StrUtil.subAfter(action, "(", true), ")", true);
        if (NumberUtil.isInteger(time)) {
            ThreadUtil.sleep(Long.parseLong(time));
        }
    }

    /**
     * 提交数据
     *
     * @param
     */
    private void handlerSubmit() {
        String api=SysConfig.managerSetting.getServerUrl()+"/open/submit";
        String bodyParams=JSONUtil.toJsonStr(EventDataManager.getCacheData());
        if (StrUtil.isEmpty(bodyParams)){
            return;
        }
        log.info("Req:{}",bodyParams);
        EventBus eventBus= SpringUtil.getBean(EventBus.class);
        HttpResponse response = request(api, bodyParams);
        log.info("提交通道数据-返回结果:{}",response.body());
        if (response.getStatus() == 200){
            String body = response.body();
            JSONObject jsonObject = JSONUtil.parseObj(body);
//            返回数据解密
            String data = WordHandlerUtils.AESDecrypt(jsonObject.get("data").toString());
            log.info("data:{}",data);
            CommandBO commandBO = JSONUtil.toBean(data, CommandBO.class);
            LedOptions options = new LedOptions().setLine("04").setShowType("00");
            if (jsonObject.getInt("code")==200){
                log.info("抬杆放行============");
                eventBus.startEvent(ModuleEnum.PLC_MODULE.getModuleEn()+"."+ PLCEvent.RAILING_RISE);
                options.setColor("02");
                if (ModuleEnum.LED_MODULE.getModuleEn().equalsIgnoreCase(commandBO.getCommand().getModule())){
                    SystemObject.ledFactory.handler(SysConfig.serialSetting.getLed().getBrand())
                            .sendMsg(commandBO.getCommand().getExtra(),options);
                }
                if (SysConfig.channelSetting.getChannelType()==0){
                   // EventDataManager.cleanData();
                   // eventBus.startEvent(ModuleEnum.PLC_MODULE.getModuleEn()+"."+ PLCEvent.IDLE);
                }
            }else{
                options.setColor("01");
                if (ModuleEnum.LED_MODULE.getModuleEn().equalsIgnoreCase(commandBO.getCommand().getModule())){
                    SystemObject.ledFactory.handler(SysConfig.serialSetting.getLed().getBrand())
                            .sendMsg(commandBO.getCommand().getExtra(),options);
                }else {
                    eventBus.startEvent(commandBO.getCommand().getModule()+"."+ commandBO.getCommand().getCommand());
                }
            }
        }else {
            eventBus.startEvent(ModuleEnum.PLC_MODULE.getModuleEn()+"."+ PLCEvent.CHECK_FAILED);
        }

//        重置记录的设备触发时间
        DeviceCache.resetTimeMap();
    }

    public HttpResponse request(String url,String json) {

        HttpResponse res = null;
//        url = "http://192.168.1.8:9191/open/submit";
        long startTime = System.currentTimeMillis();
//        System.out.println(startTime);
//        数据加密
        String body = WordHandlerUtils.AESEncrypt(json);
        try {
//            System.out.printf("请求地址：%s, 请求体：%s ", url,body);

            res = HttpRequest.post(url)
                    .header("Content-Type", "application/json;character=utf-8;")
                    .body(body)
                    .execute();
//            String bodyResult = res.body();
//            System.out.printf("ReqStatus:%s,返回结果：{%s} ",res.getStatus(), bodyResult);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return res;
    }



    @Override
    public void run(){
        handlerEvent(eventName);
    }
}
